function [ RSMbeta,PrD,alphai,pf,nhist,m ] = RSM_FORM_NLFEA( Bgrade, Sgrade, P, f )
%RSM_FORM_NLFEA calculates reliability of a beam with NLFEA input.
%Bgrade is concrete grade. Available from B15 to B45.
%Sgrade is steel grade. Available from B400NC to B600NC.
%L is length of beam in mm.
%P is point load on the middle of the beam in N.
%f is the f-factor used to calculate sample. Usually between 0.5 and 3.0
%RSMbeta is a history of converged beta for each RSM-iteration.
%PrD is design point, concrete then steel.
%alphai is influence factors. Concrete then steel.
%pf is probability of failure.
%nhist is the history of FORM-iterations per RSM-iteration.
%m is the number of RSM-iterations.


%FORM convergence demand is set at epsilonFORM. Suggested value 0.1%.
epsilonFORM = 0.001;

%RSM convergence demand is set at epsilonRSM. Suggested value 1%.
epsilonRSM = 0.01;

%Calculates parameters for steel in accordance with JCSS.
muY = Sgrade+60;
sigmaY = 23.4;

%General JCSS concrete intermediary parameters.
grade = [15, 25, 35, 45];
mp = [3.4, 3.65, 3.85, 3.98];
np = 3;
sp = [0.14,0.12,0.09,0.07];
vp = 10;

%Finding specific JCSS concrete intermediary parameters.
if Bgrade < grade(1);
    fprintf('/nFor lav betongkvalitet./n/n');
    return
elseif Bgrade < grade(2)
    mp = (Bgrade-grade(1))*(mp(2)-mp(1))/(grade(2)-grade(1))+mp(1);
    sp = (Bgrade-grade(1))*(sp(2)-sp(1))/(grade(2)-grade(1))+sp(1);
elseif Bgrade < grade(3)
    mp = (Bgrade-grade(2))*(mp(3)-mp(2))/(grade(3)-grade(2))+mp(2);
    sp = (Bgrade-grade(2))*(sp(3)-sp(2))/(grade(3)-grade(2))+sp(2);
elseif Bgrade < grade(4)
    mp = (Bgrade-grade(3))*(mp(4)-mp(3))/(grade(4)-grade(3))+mp(3);
    sp = (Bgrade-grade(3))*(sp(4)-sp(3))/(grade(4)-grade(3))+sp(3);
elseif Bgrade == grade(4)
    mp = mp(4);
    sp = sp(4);
elseif Bgrade > grade(4)
    fprintf('/nFor hy betongkvalitet/n/n');
    return
else
    fprintf('/nUspesifisert feil');
    return
end

%Calculates concrete cylinder parameters.
muCO = mp;
sigmaCO = sp*sqrt((np/(np-1)*vp/(vp-2)));

%Calculates in-situ parameters.
muC = muCO/1.1114;
sigmaC = sigmaCO/1.0417;

%Calculates first RSM-iteration center point in real space.
Pr0 = [exp(muC),muY];

%Prepares start-values for RSM-iteration.
RSMconverge = 0;
m = 0;
RSMbeta = 0;
Pr0hist = Pr0;

%Does RSM-iteration.
while RSMconverge == 0
    %Counts number of RSM-iterations.
    m = m+1;

    %Calculates center point in natural space.
    Pn0 = [(log(Pr0(1))-muC)/sigmaC, (Pr0(2)-muY)/sigmaY];

    %Calculates natural sampling point matrix.
    Pn = [Pn0 ; Pn0(1)-f,Pn0(2) ; Pn0(1)+f,Pn0(2) ; Pn0(1),Pn0(2)-f ; Pn0(1),Pn0(2)+f];

    %Calculates real sampling point matrix.
    Pr = Pn;
    Pr(:,1) = exp(Pn(:,1).*sigmaC+muC);
    Pr(:,2) = Pn(:,2).*sigmaY+muY;
    
    %Expresses sampling point matrix, asks for respective capacity vector.
    Pr
    Rr = input('\nPr is the variable input matrix for Diana. Please insert corresponding Diana load resistance vector, Rr, in [N]:\n\n');
    
    %Calculates LSF vector.
    Gr(:,1) = Rr-P;

    %Creates the D-matrix from D*d=Gr.
    [r,k] = size(Pn);
    D = zeros(r);
    D(:,1) = 1;
    D(:,2:(k+1)) = Pn;
    D(:,(k+2):r) = Pn.^2;

    %Calculates the second order polynomial Responce Surface vector, d.
    d = D\Gr;
    
    %Prepares start-values for FORM-iteration.
    FORMconverge = 0;
    n = 0;
    FORMbetahist = 0;
    uistar = Pn0;

    %Does FORM-iteration.
    while FORMconverge == 0
        %Counts number of FORM-iterations.
        n = n+1;
        
        %Creates the partial derivated D-matrix at given center point.
        ld = length(d);
        l = length(Pn0);
        dD = zeros(l,ld);
        dD(:,2:l+1) = eye(l);
        dD(:,(l+2):ld) = 2*eye(l);
        for i = 1:l
            dD(i,(l+2):ld) = dD(i,(l+2):ld)*Pn0(i);
        end
        
        %Creates linear coefficients ai by ai = dG = dD*d.
        ai = dD*d;
        
        %Adds up the sums making a0-coefficient.
        a01 = d(1) + sum(Pn0*d(2:l+1)) + sum((Pn0).^2*d(l+2:ld));
        a02 = Pn0*ai;
        a0 = a01 - a02 ;
        
        %Creates linear coefficient vector of G(Ui) = a0+sum(ai*Ui).
        coeff = [a0;ai];

        %Calculates nonlinear HL parameters for multiple variables.
        muG = coeff(1);
        sigmaG = sqrt(sum(coeff(2:end).^2));
        FORMbetatemp = muG/sigmaG;
        alphai = coeff(2:end)/sigmaG;
        alphai = alphai';
        Pn0 = -alphai*FORMbetatemp;
        
        %Creates history of FORM-betas for convergence checks.       
        FORMbetahist(n) = FORMbetatemp;
        
        %Checks whether it's the first FORM-beta, and checks convergence.
        l = length(FORMbetahist);
        if l == 1
            FORMconverge = 0;
        else
            epsilon = abs((FORMbetahist(l)-FORMbetahist(l-1))/FORMbetahist(l));
            if epsilon < epsilonFORM;
                FORMconverge = 1;
            else
                FORMconverge = 0;
            end
        end
    end
    
    %Writes the history of FORM-betas for this RSM-iteration.
    FORMbetahist
    
    %Collects the last iterated beta-value.
    RSMbeta(m,1) = FORMbetahist(end);
    
    %Transforms FORM-iterated design point into real space.
    Pr0(1,1) = exp(Pn0(1)*sigmaC+muC);
    Pr0(1,2) = muY+Pn0(2)*sigmaY;

    %Holds the history of Pr0.
    Pr0hist = [Pr0hist;Pr0];
    
    %Checks whether it's the first RSM-beta, and checks convergence.
    l = length(RSMbeta);
    if l == 1
        RSMconverge = 0;
    else
        epsilon = abs((RSMbeta(l)-RSMbeta(l-1))/RSMbeta(l));
        if epsilon < epsilonRSM;
            RSMconverge = 1;
        else
            RSMconverge = 0;
        end
    end
    
    %Logs number of FORM-iterations.
    nhist(m) = n;
end

%Defines the design point and probability of failure.
PrD = Pr0(end,:);
pf = normcdf(-RSMbeta(end));
end